# AttributionLab
## Introduction
This is the official implementation of "*AttributionLab: Faithfulness of Feature Attribution Under Controllable Environments*".
## Installation
1. Make sure that you are in the root of the project. I.e. being at the same level as `pyproject.toml`.
2. `pip install -e .` (Do not forget the ".") For upgrading the package please use `pip install -U -e .`  .

## Structure overview
* configs: configuration for some scripts
* scripts: executable files
* sde: root directory for the project
  - attribution_methods: attribution methods for feature attribution
  - dataset_generators: generator classes to create synthetic datasets
  - datasets: various dataset classes used in the project
  - evaluation: code for feature attribution evaluation
  - models: synthetic models
* tests: unit tests

## Naming conventions
* The single-color-modulos setting in the paper is denoted with `black_white_modulo`, and 
multi-color-sum is denoted with `multi_color_sum` in the config files under `configs/` folder.
* For the accumulator module, there are two weight assignment (or initialization) schemes: uniform and non-uniform. 
The configs associated with non-uniform weight assignment have suffix `_non_uniform`. 

## Example usage
* Synthesize a black-white-modulo dataset (with patches surrounded by Bézier curves):
  ```bash
  python scripts/dataset_generation/make_dataset.py 
    configs/dataset_generation/black_white_bezier.yml
  ```

* Test the synthetic (i.e. programmed) model with non-uniform weight assignment on the black-white-modulo dataset:
  ```bash
  python scripts/eval_synthetic_setup.py 
    configs/eval_synthetic_setup/black_white_bezier_non_uniform.yml
  ```
  Since the model is designed to perform the task, the test accuracy should always be `100%`. 

* Run feature attribution on the black-white-modulo dataset, using a synthetic model with 
non-uniform weight assignment. 
  ```bash
  python scripts/generate_attribution_maps.py 
    configs/attribution/attribution_black_white_modulo_non_uniform.yml 
    -o attribution_maps/black_white_modulo/bezier/
  ```

* Run Deletion on the GradCAM attribution maps on the black-white-modulo dataset:
  ```bash
  python scripts/attr_evaluation/eval_attribution_maps.py 
    configs/benchmarks/ins_del_black_white_modulo_non_uniform.yml 
    -s 100
    -o workdirs/gradcam/deletion/
  ```
  The `-s 100` means to select a subset of 100 samples. It helps to reduce the time consumption.

  To run Insertion, you just need to modify the config from the command line:
  ```bash
  python scripts/attr_evaluation/eval_attribution_maps.py 
    configs/benchmarks/ins_del_black_white_modulo_non_uniform.yml 
    -s 100
    -o workdirs/gradcam/insertion/
    --cfg-options evaluator.mode=insertion
  ```
  To run other metrics, please switch to another config under `configs/benchmarks/`.
