from typing import Tuple

import numpy as np
import torch
import torch.nn as nn
from torchray.attribution.extremal_perturbation import extremal_perturbation

from ..utils import check_shape, norm_and_to_uint8


def compute_extremal_perturbation(
        model: nn.Module,
        preprocessed_image: torch.Tensor,
        label: torch.Tensor,
        need_check_shape: bool = False) -> Tuple[np.ndarray, np.ndarray]:
    if need_check_shape:
        if not check_shape(preprocessed_image.shape, '1,c,n,n'):
            raise ValueError(f'image has invalid shape: {preprocessed_image.shape}')
    # saliency shape: (1, 1, height ,width)
    saliency, _ = extremal_perturbation(model, preprocessed_image, label.item())
    saliency = saliency.detach().cpu().numpy().squeeze(0)
    saliency = saliency.mean(0)
    normed_saliency = norm_and_to_uint8(saliency, norm='abs_zero_max')

    if need_check_shape:
        if not check_shape(saliency.shape, 'n,n'):
            raise ValueError(f'saliency has invalid shape: {saliency.shape}')
    return saliency.astype(np.float16), normed_saliency
