from typing import Optional, Tuple

import numpy as np
import torch
import torch.nn as nn

from ..utils import check_shape, norm_and_to_uint8


def compute_random_mask(
        model: nn.Module,
        preprocessed_image: torch.Tensor,
        label: torch.Tensor,
        baseline: Optional[str] = None,
        need_check_shape: bool = False) -> Tuple[np.ndarray, np.ndarray]:
    if need_check_shape:
        if not check_shape(preprocessed_image.shape, '1,c,n,n'):
            raise ValueError(f'image has invalid shape: {preprocessed_image.shape}')

    grad = np.random.rand(*preprocessed_image.shape[-2:])
    normed_grad = norm_and_to_uint8(grad, norm='abs_zero_max')

    if need_check_shape:
        if not check_shape(grad.shape, 'n,n'):
            raise ValueError(f'grad has invalid shape: {grad.shape}')
    return grad.astype(np.float16), normed_grad
