from copy import deepcopy
from typing import Dict, Optional

from .attr_percentage import AttributionPercentage
from .base import BaseEvaluation
from .gt_evaluation import GroundTruthEvaluation
from .insertion_deletion import InsertionDeletion
from .sensitivity_n import SensitivityN

EVALUATOR_CATALOG = {
    'InsertionDeletion': InsertionDeletion,
    'SensitivityN': SensitivityN,
    'AttributionPercentage': AttributionPercentage,
    'GroundTruthEvaluation': GroundTruthEvaluation
}


def build_evaluator(cfg: Dict, default_args: Optional[Dict] = None) -> BaseEvaluation:
    _cfg = deepcopy(cfg)
    if default_args is not None:
        _cfg.update(default_args)

    evaluator_type = _cfg.pop('type')
    if evaluator_type in EVALUATOR_CATALOG:
        evaluator = EVALUATOR_CATALOG[evaluator_type](**_cfg)
    else:
        raise NotImplementedError(f'Not supported evaluator type: {evaluator_type}')

    return evaluator
